@@ -1,18 +1,18 @@
 /***************************************************************************
-* Copyright (c) Johan Mabille, Sylvain Corlay and Wolf Vollprecht          *
-* Copyright (c) QuantStack                                                 *
-*                                                                          *
-* Distributed under the terms of the BSD 3-Clause License.                 *
-*                                                                          *
-* The full license is in the file LICENSE, distributed with this software. *
-****************************************************************************/
+ * Copyright (c) Johan Mabille, Sylvain Corlay and Wolf Vollprecht          *
+ * Copyright (c) QuantStack                                                 *
+ *                                                                          *
+ * Distributed under the terms of the BSD 3-Clause License.                 *
+ *                                                                          *
+ * The full license is in the file LICENSE, distributed with this software. *
+ ****************************************************************************/
 
 #ifndef XOPTIONAL_ASSEMBLY_BASE_HPP
 #define XOPTIONAL_ASSEMBLY_BASE_HPP
 
 #include "xiterable.hpp"
-#include "xtensor_forward.hpp"
 #include "xoptional_assembly_storage.hpp"
+#include "xtensor_forward.hpp"
 
 namespace xt
 {
@@ -105,10 +105,10 @@ namespace xt
         template <class S, layout_type L>
         using const_reverse_broadcast_iterator = typename iterable_base::template const_reverse_broadcast_iterator<S, L>;
 
-        using storage_iterator = typename storage_type::iterator;
-        using const_storage_iterator = typename storage_type::const_iterator;
-        using reverse_storage_iterator = typename storage_type::reverse_iterator;
-        using const_reverse_storage_iterator = typename storage_type::const_reverse_iterator;
+        using linear_iterator = typename storage_type::iterator;
+        using const_linear_iterator = typename storage_type::const_iterator;
+        using reverse_linear_iterator = typename storage_type::reverse_iterator;
+        using const_reverse_linear_iterator = typename storage_type::const_reverse_iterator;
 
         using iterator = typename iterable_base::iterator;
         using const_iterator = typename iterable_base::const_iterator;
@@ -130,10 +130,10 @@ namespace xt
         void resize(const S& shape, const strides_type& strides);
 
         template <class S = shape_type>
-        auto & reshape(const S& shape, layout_type layout = static_layout) &;
+        auto& reshape(const S& shape, layout_type layout = static_layout) &;
 
         template <class T>
-        auto & reshape(std::initializer_list<T> shape, layout_type layout = static_layout) &;
+        auto& reshape(std::initializer_list<T> shape, layout_type layout = static_layout) &;
 
         layout_type layout() const noexcept;
         bool is_contiguous() const noexcept;
@@ -177,6 +177,15 @@ namespace xt
         template <class... Args>
         const_reference periodic(Args... args) const;
 
+        reference front();
+        const_reference front() const;
+
+        reference back();
+        const_reference back() const;
+
+        reference flat(size_type args);
+        const_reference flat(size_type args) const;
+
         template <class It>
         reference element(It first, It last);
         template <class It>
@@ -199,29 +208,29 @@ namespace xt
         bool has_linear_assign(const S& strides) const noexcept;
 
         using iterable_base::begin;
-        using iterable_base::end;
         using iterable_base::cbegin;
         using iterable_base::cend;
-        using iterable_base::rbegin;
-        using iterable_base::rend;
         using iterable_base::crbegin;
         using iterable_base::crend;
+        using iterable_base::end;
+        using iterable_base::rbegin;
+        using iterable_base::rend;
 
-        storage_iterator storage_begin() noexcept;
-        storage_iterator storage_end() noexcept;
+        linear_iterator linear_begin() noexcept;
+        linear_iterator linear_end() noexcept;
 
-        const_storage_iterator storage_begin() const noexcept;
-        const_storage_iterator storage_end() const noexcept;
-        const_storage_iterator storage_cbegin() const noexcept;
-        const_storage_iterator storage_cend() const noexcept;
+        const_linear_iterator linear_begin() const noexcept;
+        const_linear_iterator linear_end() const noexcept;
+        const_linear_iterator linear_cbegin() const noexcept;
+        const_linear_iterator linear_cend() const noexcept;
 
-        reverse_storage_iterator storage_rbegin() noexcept;
-        reverse_storage_iterator storage_rend() noexcept;
+        reverse_linear_iterator linear_rbegin() noexcept;
+        reverse_linear_iterator linear_rend() noexcept;
 
-        const_reverse_storage_iterator storage_rbegin() const noexcept;
-        const_reverse_storage_iterator storage_rend() const noexcept;
-        const_reverse_storage_iterator storage_crbegin() const noexcept;
-        const_reverse_storage_iterator storage_crend() const noexcept;
+        const_reverse_linear_iterator linear_rbegin() const noexcept;
+        const_reverse_linear_iterator linear_rend() const noexcept;
+        const_reverse_linear_iterator linear_crbegin() const noexcept;
+        const_reverse_linear_iterator linear_crend() const noexcept;
 
         template <class S>
         stepper stepper_begin(const S& shape) noexcept;
@@ -271,22 +280,17 @@ namespace xt
         using self_type = xoptional_assembly_stepper<D, is_const>;
         using assembly_type = typename D::assembly_type;
         using value_type = typename assembly_type::value_type;
-        using reference = std::conditional_t<is_const,
-                                             typename assembly_type::const_reference,
-                                             typename assembly_type::reference>;
-        using pointer = std::conditional_t<is_const,
-                                           typename assembly_type::const_pointer,
-                                           typename assembly_type::pointer>;
+        using reference = std::
+            conditional_t<is_const, typename assembly_type::const_reference, typename assembly_type::reference>;
+        using pointer = std::conditional_t<is_const, typename assembly_type::const_pointer, typename assembly_type::pointer>;
         using size_type = typename assembly_type::size_type;
         using difference_type = typename assembly_type::difference_type;
         using raw_value_expression = typename assembly_type::raw_value_expression;
         using raw_flag_expression = typename assembly_type::raw_flag_expression;
-        using value_stepper = std::conditional_t<is_const,
-                                                 typename raw_value_expression::const_stepper,
-                                                 typename raw_value_expression::stepper>;
-        using flag_stepper = std::conditional_t<is_const,
-                                                typename raw_flag_expression::const_stepper,
-                                                typename raw_flag_expression::stepper>;
+        using value_stepper = std::
+            conditional_t<is_const, typename raw_value_expression::const_stepper, typename raw_value_expression::stepper>;
+        using flag_stepper = std::
+            conditional_t<is_const, typename raw_flag_expression::const_stepper, typename raw_flag_expression::stepper>;
 
         xoptional_assembly_stepper(value_stepper vs, flag_stepper fs) noexcept;
 
@@ -330,7 +334,7 @@ namespace xt
      * Returns the number of dimensions of the optional assembly.
      */
     template <class D>
-    inline auto constexpr xoptional_assembly_base<D>::dimension() const noexcept -> size_type
+    inline constexpr auto xoptional_assembly_base<D>::dimension() const noexcept -> size_type
     {
         return value().dimension();
     }
@@ -370,6 +374,7 @@ namespace xt
     {
         return value().backstrides();
     }
+
     //@}
 
     /**
@@ -418,7 +423,7 @@ namespace xt
      */
     template <class D>
     template <class S>
-    inline auto & xoptional_assembly_base<D>::reshape(const S& shape, layout_type layout) &
+    inline auto& xoptional_assembly_base<D>::reshape(const S& shape, layout_type layout) &
     {
         value().reshape(shape, layout);
         has_value().reshape(shape, layout);
@@ -427,7 +432,7 @@ namespace xt
 
     template <class D>
     template <class T>
-    inline auto & xoptional_assembly_base<D>::reshape(std::initializer_list<T> shape, layout_type layout) &
+    inline auto& xoptional_assembly_base<D>::reshape(std::initializer_list<T> shape, layout_type layout) &
     {
         value().reshape(shape, layout);
         has_value().reshape(shape, layout);
@@ -458,7 +463,7 @@ namespace xt
     template <class T>
     inline void xoptional_assembly_base<D>::fill(const T& value)
     {
-        std::fill(this->storage_begin(), this->storage_end(), value);
+        std::fill(this->linear_begin(), this->linear_end(), value);
     }
 
     /**
@@ -583,16 +588,14 @@ namespace xt
      */
     template <class D>
     template <class S>
-    inline auto xoptional_assembly_base<D>::operator[](const S& index)
-        -> disable_integral_t<S, reference>
+    inline auto xoptional_assembly_base<D>::operator[](const S& index) -> disable_integral_t<S, reference>
     {
         return reference(value()[index], has_value()[index]);
     }
 
     template <class D>
     template <class I>
-    inline auto xoptional_assembly_base<D>::operator[](std::initializer_list<I> index)
-        -> reference
+    inline auto xoptional_assembly_base<D>::operator[](std::initializer_list<I> index) -> reference
     {
         return reference(value()[index], has_value()[index]);
     }
@@ -619,8 +622,7 @@ namespace xt
 
     template <class D>
     template <class I>
-    inline auto xoptional_assembly_base<D>::operator[](std::initializer_list<I> index) const
-        -> const_reference
+    inline auto xoptional_assembly_base<D>::operator[](std::initializer_list<I> index) const -> const_reference
     {
         return const_reference(value()[index], has_value()[index]);
     }
@@ -659,6 +661,64 @@ namespace xt
         return const_reference(value().periodic(args...), has_value().periodic(args...));
     }
 
+    /**
+     * Returns a reference to the first element of the optional assembly.
+     */
+    template <class D>
+    inline auto xoptional_assembly_base<D>::front() -> reference
+    {
+        return reference(value().front(), has_value().front());
+    }
+
+    /**
+     * Returns a constant reference to the first element of the optional assembly.
+     */
+    template <class D>
+    inline auto xoptional_assembly_base<D>::front() const -> const_reference
+    {
+        return const_reference(value().front(), has_value().front());
+    }
+
+    /**
+     * Returns a reference to the last element of the optional assembly.
+     */
+    template <class D>
+    inline auto xoptional_assembly_base<D>::back() -> reference
+    {
+        return reference(value().back(), has_value().back());
+    }
+
+    /**
+     * Returns a constant reference to the last element of the optional assembly.
+     */
+    template <class D>
+    inline auto xoptional_assembly_base<D>::back() const -> const_reference
+    {
+        return const_reference(value().back(), has_value().back());
+    }
+
+    /**
+     * Returns a reference to the element at the specified position
+     * of the underlying storage in the optional assembly.
+     * @param index index to underlying flat storage.
+     */
+    template <class D>
+    inline auto xoptional_assembly_base<D>::flat(size_type i) -> reference
+    {
+        return reference(value().flat(i), has_value().flat(i));
+    }
+
+    /**
+     * Returns a constant reference to the element at the specified position
+     * of the underlying storage in the optional assembly.
+     * @param index index to underlying flat storage.
+     */
+    template <class D>
+    inline auto xoptional_assembly_base<D>::flat(size_type i) const -> const_reference
+    {
+        return const_reference(value().flat(i), has_value().flat(i));
+    }
+
     /**
      * Returns a reference to the element at the specified position in the optional assembly.
      * @param first iterator starting the sequence of indices
@@ -698,6 +758,7 @@ namespace xt
     {
         return value().in_bounds(args...) && has_value().in_bounds(args...);
     }
+
     //@}
 
     template <class D>
@@ -759,82 +820,79 @@ namespace xt
     {
         return value().has_linear_assign(strides) && has_value().has_linear_assign(strides);
     }
+
     //@}
 
     template <class D>
-    inline auto xoptional_assembly_base<D>::storage_begin() noexcept -> storage_iterator
+    inline auto xoptional_assembly_base<D>::linear_begin() noexcept -> linear_iterator
     {
-        return storage_iterator(value().storage_begin(),
-                                has_value().storage_begin());
+        return linear_iterator(value().linear_begin(), has_value().linear_begin());
     }
 
     template <class D>
-    inline auto xoptional_assembly_base<D>::storage_end() noexcept -> storage_iterator
+    inline auto xoptional_assembly_base<D>::linear_end() noexcept -> linear_iterator
     {
-        return storage_iterator(value().storage_end(),
-                                has_value().storage_end());
+        return linear_iterator(value().linear_end(), has_value().linear_end());
     }
 
     template <class D>
-    inline auto xoptional_assembly_base<D>::storage_begin() const noexcept -> const_storage_iterator
+    inline auto xoptional_assembly_base<D>::linear_begin() const noexcept -> const_linear_iterator
     {
-        return storage_cbegin();
+        return linear_cbegin();
     }
 
     template <class D>
-    inline auto xoptional_assembly_base<D>::storage_end() const noexcept -> const_storage_iterator
+    inline auto xoptional_assembly_base<D>::linear_end() const noexcept -> const_linear_iterator
     {
-        return storage_cend();
+        return linear_cend();
     }
 
     template <class D>
-    inline auto xoptional_assembly_base<D>::storage_cbegin() const noexcept -> const_storage_iterator
+    inline auto xoptional_assembly_base<D>::linear_cbegin() const noexcept -> const_linear_iterator
     {
-        return const_storage_iterator(value().storage_cbegin(),
-                                      has_value().storage_cbegin());
+        return const_linear_iterator(value().linear_cbegin(), has_value().linear_cbegin());
     }
 
     template <class D>
-    inline auto xoptional_assembly_base<D>::storage_cend() const noexcept -> const_storage_iterator
+    inline auto xoptional_assembly_base<D>::linear_cend() const noexcept -> const_linear_iterator
     {
-        return const_storage_iterator(value().storage_cend(),
-                                      has_value().storage_cend());
+        return const_linear_iterator(value().linear_cend(), has_value().linear_cend());
     }
 
     template <class D>
-    inline auto xoptional_assembly_base<D>::storage_rbegin() noexcept -> reverse_storage_iterator
+    inline auto xoptional_assembly_base<D>::linear_rbegin() noexcept -> reverse_linear_iterator
     {
-        return reverse_storage_iterator(storage_end());
+        return reverse_linear_iterator(linear_end());
     }
 
     template <class D>
-    inline auto xoptional_assembly_base<D>::storage_rend() noexcept -> reverse_storage_iterator
+    inline auto xoptional_assembly_base<D>::linear_rend() noexcept -> reverse_linear_iterator
     {
-        return reverse_storage_iterator(storage_begin());
+        return reverse_linear_iterator(linear_begin());
     }
 
     template <class D>
-    inline auto xoptional_assembly_base<D>::storage_rbegin() const noexcept -> const_reverse_storage_iterator
+    inline auto xoptional_assembly_base<D>::linear_rbegin() const noexcept -> const_reverse_linear_iterator
     {
-        return storage_crbegin();
+        return linear_crbegin();
     }
 
     template <class D>
-    inline auto xoptional_assembly_base<D>::storage_rend() const noexcept -> const_reverse_storage_iterator
+    inline auto xoptional_assembly_base<D>::linear_rend() const noexcept -> const_reverse_linear_iterator
     {
-        return storage_crend();
+        return linear_crend();
     }
 
     template <class D>
-    inline auto xoptional_assembly_base<D>::storage_crbegin() const noexcept -> const_reverse_storage_iterator
+    inline auto xoptional_assembly_base<D>::linear_crbegin() const noexcept -> const_reverse_linear_iterator
     {
-        return const_reverse_storage_iterator(storage_cend());
+        return const_reverse_linear_iterator(linear_cend());
     }
 
     template <class D>
-    inline auto xoptional_assembly_base<D>::storage_crend() const noexcept -> const_reverse_storage_iterator
+    inline auto xoptional_assembly_base<D>::linear_crend() const noexcept -> const_reverse_linear_iterator
     {
-        return const_reverse_storage_iterator(storage_cbegin());
+        return const_reverse_linear_iterator(linear_cbegin());
     }
 
     template <class D>
@@ -860,7 +918,8 @@ namespace xt
 
     template <class D>
     template <class S>
-    inline auto xoptional_assembly_base<D>::stepper_end(const S& shape, layout_type l) const noexcept -> const_stepper
+    inline auto xoptional_assembly_base<D>::stepper_end(const S& shape, layout_type l) const noexcept
+        -> const_stepper
     {
         return const_stepper(value().stepper_end(shape, l), has_value().stepper_end(shape, l));
     }
@@ -919,7 +978,8 @@ namespace xt
 
     template <class D, bool C>
     inline xoptional_assembly_stepper<D, C>::xoptional_assembly_stepper(value_stepper vs, flag_stepper fs) noexcept
-        : m_vs(vs), m_fs(fs)
+        : m_vs(vs)
+        , m_fs(fs)
     {
     }
 
